


#include <iostream>
#include <cmath>

#include "HLayoutContainer.h"
#include "HSplitter.h"


// Start of CEGUI namespace section
namespace CEGUI
{
	const String HLayoutContainer::EventNamespace("HLayoutContainer");
	const String HLayoutContainer::WidgetTypeName("CEGUI/HLayoutContainer");

	HLayoutContainer::HLayoutContainer(const String& type, const String& name) :
		SequentialLayoutContainer(type, name)
	{
		splitterWidth = 15.0f;


		subscribeEvent(Window::EventSized, Event::Subscriber(&HLayoutContainer::handleSized, this));

		// we have to subscribe to the EventSized for layout updates
		//d_eventConnections.insert(std::make_pair(this,
		//	this->subscribeEvent(Window::EventSized,
		//	Event::Subscriber(&LayoutContainer::handleChildSized, this))));
	}


	/*************************************************************************
	Destructor
	*************************************************************************/
	HLayoutContainer::~HLayoutContainer(void)
	{
	}


	void HLayoutContainer::layout()
	{
		if (bNoRelayout)
		{
			//std::cout << "NO layout\n";
			bNoRelayout = false;
			return;
		}


		std::cout << "make layout\n";

		float totalChildrenWidth = 0.0f;
		float totalSplitterWidth = 0.0f;

		for (size_t i = 0; i < d_children.size(); i++)
		{
			Window* wnd = getChildAtIdx(i);
			totalChildrenWidth += wnd->getPixelSize().d_width;
			HSplitter* spl = dynamic_cast<HSplitter*>(wnd);

			//std::cout << "min sz:" << wnd->getMinSize();

			if (spl)
			{
				//std::cout << "split\n";

				totalSplitterWidth += getChildAtIdx(i)->getPixelSize().d_width;
			}
			else
			{
				//std::cout << "not split\n";

			}
		}
		//std::cout << "child window width:" << totalChildrenWidth - totalSplitterWidth << "\n";
		//std::cout << "available window width:" << getPixelSize().d_width - totalSplitterWidth << "\n";
		float childWindowScale = (getPixelSize().d_width - totalSplitterWidth) / (totalChildrenWidth - totalSplitterWidth);
		//std::cout << "child scale:" << childWindowScale << "\n";


		float nextChildPosition = 0.0f;
		float curChildWidth = 0.0f;

		for (size_t i = 0; i < d_children.size(); i++)
		{
			Window* wnd = getChildAtIdx(i);
			HSplitter* spl = dynamic_cast<HSplitter*>(wnd);

			wnd->setXPosition(UDim(0.0f, nextChildPosition));

			curChildWidth = wnd->getPixelSize().d_width;

			if (!spl)//if this is not a splitter - scale it appropriately
			{
				curChildWidth *= childWindowScale;

				if (curChildWidth < windowMinSize)
					curChildWidth = windowMinSize;

				wnd->setWidth(UDim(0.0f, curChildWidth));
				//std::cout << "not split\n";

			}
			nextChildPosition += curChildWidth;
		}
		
	}


	void HLayoutContainer::swapChildPositions(size_t wnd1, size_t wnd2)
	{
		if (wnd1 == wnd2)
			return;

		//convert to real positions
		wnd1 *= 2;
		wnd2 *= 2;

		size_t numChildren = d_children.size();

		//we need to make sure that wnd1 comes before wnd2
		if (wnd1 > wnd2)
		{
			size_t temp = wnd1;
			wnd1 = wnd2;
			wnd2 = temp;
		}

		if (wnd1 < numChildren && wnd2 < numChildren)
		{
			std::swap(d_children[wnd1], d_children[wnd2]);

			//suppose we have order of children as follows (we made sure that wnd1 < wnd2):
			//... lspl1 wnd1 rspl1 ... lspl2 wnd2 rspl2 ...
			//we need to fix pointers of splitters lspl1, rspl1, lspl2, rspl2

			//wnd1 rspl1==lspl2 wnd2 rspl2

			HSplitter* lspl1 = 0;
			HSplitter* rspl1 = 0;
			HSplitter* lspl2 = 0;
			HSplitter* rspl2 = 0;


			if (wnd1 > 0)
				lspl1 = dynamic_cast<HSplitter*>(d_children[wnd1 - 1]);

			if ((wnd1 + 1) < numChildren)
				rspl1 = dynamic_cast<HSplitter*>(d_children[wnd1 + 1]);

			if (wnd2 > 0)
				lspl2 = dynamic_cast<HSplitter*>(d_children[wnd2 - 1]);

			if ((wnd2 + 1) < numChildren)
				rspl2 = dynamic_cast<HSplitter*>(d_children[wnd2 + 1]);

			//set correct pointers
			if (lspl1)
				lspl1->setNextWindow(dynamic_cast<Window*>(d_children[wnd1]));
			if (rspl1)
				rspl1->setPrevWindow(dynamic_cast<Window*>(d_children[wnd1]));

			if (lspl2)
				lspl2->setNextWindow(dynamic_cast<Window*>(d_children[wnd2]));
			if (rspl2)
				rspl2->setPrevWindow(dynamic_cast<Window*>(d_children[wnd2]));


			WindowEventArgs args(this);
			onChildOrderChanged(args);
		}

	}

	void HLayoutContainer::moveChildToPosition(Window* wnd, size_t position)
	{
		if (!isChild(wnd))
			return;

		size_t numChildren = d_children.size();

		//convert to real position
		position *= 2;

		//if we don't have at least two windows (and one splitter)
		//then we don't want to do anything
		if (numChildren < 3)
			return;

		if (position >= numChildren)
			position = numChildren - 1;

		const size_t oldPosition = getIdxOfChild(wnd);

		//inserting window one index ahead will give nothing so skip it too
		if (oldPosition == position || oldPosition + 1 == position)
			return;

		//if we are moving some window, we'll also move splitter to the left from window
		//but if we are moving the first window then there is no splitter to the left
		//but don't forget that then we'll need to delete splitter to the right (since it can't be first child)
		//instead of deleting let's think that it was to the left :)
		size_t splitterPos = oldPosition > 0 ? oldPosition - 1 : 1;

		//splitter that we will move
		HSplitter* spl = dynamic_cast<HSplitter*>(d_children[splitterPos]);

		//basically like in sequential container
		// we get the iterator of the old position
		ChildList::iterator it = d_children.begin();

		std::cout << "wnd old: " << oldPosition << ", spl old: " << splitterPos << "\n";

		//advance to what is first (window or splitter)
		if (oldPosition < splitterPos)
		{
			std::advance(it, oldPosition);
		}
		else
		{
			std::advance(it, splitterPos);
		}

		it = d_children.erase(it);
		it = d_children.erase(it);

		std::cout << position << " asd\n";
		//fix pointer
		if (oldPosition > 0 && splitterPos > 0 && oldPosition+1 < numChildren)
		{
			HSplitter* rspl = dynamic_cast<HSplitter*>(d_children[splitterPos]);
			Window* lwnd = dynamic_cast<Window*>(d_children[splitterPos-1]);
			rspl->setPrevWindow(lwnd);
		}


		// if the window comes before the point we want to insert to,
		// we have to decrement the position
		//if (oldPosition < position)
		if (position > 0)
		{
			--position;
		}

		// find iterator of the new position
		it = d_children.begin();
		std::advance(it, position);
		// and insert splitter and window there
		it=d_children.insert(it, wnd);
		//it = d_children.begin();
		//std::advance(it, position - 1);
		
		if (position == 0)
		{
			++it;
		}

		d_children.insert(it, spl);

		//now we need to fix pointers of splitters
		//lspl0 wnd rspl0 ... lspl1 

		std::cout << position << " sdf\n";

		if (position > 0)
		{
			HSplitter* rspl = 0;
			Window* lwnd = dynamic_cast<Window*>(d_children[position - 1]);
			if (position + 2 < numChildren)
			{
				rspl = dynamic_cast<HSplitter*>(d_children[position + 2]);
				if (rspl)
				rspl->setPrevWindow(wnd);
			}

			spl->setPrevWindow(lwnd);
			spl->setNextWindow(wnd);
		}
		else
		{
			std::cout << "sdf\n";
			spl->setPrevWindow(wnd);
			Window* rwnd = dynamic_cast<Window*>(d_children[2]);
			spl->setNextWindow(rwnd);
		}


		WindowEventArgs args(this);
		onChildOrderChanged(args);

	}

	Window* HLayoutContainer::getLastWindow()
	{
		Window* wnd = 0;

		if (d_children.size() > 0)
		{
			wnd = getChildAtIdx(d_children.size() - 1);
		}

		return wnd;
	}

	HSplitter* HLayoutContainer::getLastSplitter()
	{
		HSplitter* spl = 0;

		if (d_children.size() > 1)
		{
			//last child is window. and last splitter is child before last
			spl = dynamic_cast<HSplitter*>(getChildAtIdx(d_children.size() - 2));
		}

		return spl;
	}

	float HLayoutContainer::getChildrenMinWidth()
	{
		int numWindows = (d_children.size()+1)/2;

		if (numWindows > 0)
		{
			int numSplitters = numWindows - 1;

			return numWindows*windowMinSize + numSplitters*splitterWidth;
		}

		return 0.0f;
	}

// 	void HLayoutContainer::onChildOrderChanged(WindowEventArgs& e)
// 	{
// 
// 	}

	void HLayoutContainer::handleSized(const EventArgs& e)
	{
		//std::cout << "sied\n";
		markNeedsLayouting();
	}

	void HLayoutContainer::addChild_impl(Element* element)
	{
		Window* window = dynamic_cast<Window*>(element);

		if (!window)
			CEGUI_THROW(InvalidRequestException(
			"Window can only have Elements of type Window added as children "
			"(Window path: " + getNamePath() + ")."));

		// if the element is already a child of this Window, this is a NOOP
		if (isChild(element))
			return;

		//if that was normal call to addChild then we need to add
		//extra splitter and only then add window
		if (!bAddingSplitter)
		{

			//also wee need to check that we already have at least one children so we don't
			//create unnecessary splitter
			if (d_children.size() > 0)
			{
				//std::cout << "add splitter\n";
				bAddingSplitter = true;

				HSplitter* spl = getLastSplitter();
				Window* wnd = getLastWindow();

				
				CEGUI::HSplitter* splitter = static_cast<CEGUI::HSplitter*>(CEGUI::WindowManager::getSingleton().createWindow("TaharezLook/HSplitter"));

				splitter->setPosition(CEGUI::UVector2(CEGUI::UDim(0.0f, 0.0f), CEGUI::UDim(0.0f, 0.0f)));
				splitter->setSize(CEGUI::USize(CEGUI::UDim(0.0f, splitterWidth), CEGUI::UDim(1.0f, 0.0f)));//stretch splitter vertically


				this->addChild(splitter);
				splitter->setEWSizingCursorImage("TaharezLook/MouseEsWeCursor");
				bAddingSplitter = false;

				splitter->setNextWindow(dynamic_cast<Window*>(element));

				if (spl)
				{
					//std::cout << "last splitter" << spl->getName() << "\n";
					//splitter->setPrevSplitter(spl);
					//spl->setNextSplitter(splitter);
				}
				if (wnd)
				{
					splitter->setPrevWindow(wnd);

					//std::cout << "last window" << wnd->getName() << "\n";
				}
			}
		}

		LayoutContainer::addChild_impl(window);

		//TODO we don't really want to change min size of parent
		USize prevMinSize = d_parent->getMinSize();
		prevMinSize.d_width.d_scale = 0;
		prevMinSize.d_width.d_offset = getChildrenMinWidth();
		d_parent->setMinSize(prevMinSize);
	}

	void HLayoutContainer::removeChild_impl(Element* element)
	{
		Window* window = static_cast<Window*>(element);

		if (!window)
			return;


		// if the element is not a child of this Window, just quit
		if (!isChild(element))
			return;

		int childIdx = getIdxOfChild(window);

		//this means we have at least one splitter in children list
		//so we need to remove one unneeded splitter
		if (childIdx % 2 == 0 && getChildCount() > 1)
		{
			std::cout << "remove child window: " << childIdx << "\n";
			HSplitter* splitterToRemove = 0;
			if (childIdx==getChildCount()-1)//if this is last child
			{
				std::cout << "removing last window\n";

				//we need to grab last splitter and remove it too
				splitterToRemove = static_cast<HSplitter*>(getChildAtIdx(childIdx - 1));
				if (splitterToRemove)
				{
					removeChild(splitterToRemove);
				}
			}
			else if (childIdx == 0)//first child
			{
				std::cout << "removing first window\n";

				//we need to grab first splitter and remove it too
				splitterToRemove = static_cast<HSplitter*>(getChildAtIdx(1));
				if (splitterToRemove)
				{
					removeChild(splitterToRemove);
				}
			}
			else//somewhere in between
			{
				//actually this case can be united with case when window is last,
				//but let's keep them separate for now
				std::cout << "removing intermediate window\n";


				//we need to remove left or right splitter (relatively to deleted window)
				//since this is arbitrary, lets remove left splitter

				//suppose we have following structure of children:
				//... wnd0, spl0, wnd1, spl1, wnd2, spl2, wnd3
				//and we are deleting wnd2
				//then we also delete splitter to the left (spl1)
				//after the we need to fix prev window pointer of spl2
				//prev window of spl2 should become wnd1 (instead of wnd2)

				Window* wnd1 = getChildAtIdx(childIdx - 2);
				HSplitter* spl2 = static_cast<HSplitter*>(getChildAtIdx(childIdx + 1));

				splitterToRemove = static_cast<HSplitter*>(getChildAtIdx(childIdx - 1));
				if (splitterToRemove)
				{
					removeChild(splitterToRemove);
				}

				//fix pointer of splitter to the right (spl2)
				if (wnd1 && spl2)
				{
					spl2->setPrevWindow(wnd1);
				}
			}
		}
		else if (childIdx % 2 == 1)//if we are deleting splitter
		{
			std::cout << "remove child splitter: " << childIdx << "\n";

		}

		

		LayoutContainer::removeChild_impl(window);
	}

	void HLayoutContainer::offsetSplitter(HSplitter* splitter, const Vector2f& offset)
	{
		if (!splitter)
			return;


		Window* prev = splitter->getPrevWindow();
		Window* next = splitter->getNextWindow();

		if (!prev || !next)
			return;

		bNoRelayout = true;



		float posChange = offset.d_x;




		UDim prevWidth = prev->getWidth();
		prevWidth.d_scale = 0;

		UDim nextWidth = next->getWidth();
		nextWidth.d_scale = 0;

		//check so we do not resize prev window too much
		if (posChange < 0)
		{
			if (prevWidth.d_offset + posChange < windowMinSize)
			{
				posChange = windowMinSize - prevWidth.d_offset;
			}
		}
		else// posChange > 0
		{
			if (nextWidth.d_offset - posChange < windowMinSize)
			{
				posChange = nextWidth.d_offset - windowMinSize;
			}
		}

		prevWidth.d_offset += posChange;
		prev->setWidth(prevWidth);

		nextWidth.d_offset -= posChange;
		next->setWidth(nextWidth);

		UDim nextPos = next->getXPosition();
		nextPos.d_scale = 0;
		nextPos.d_offset += posChange;

		next->setXPosition(nextPos);


		UVector2 uOffset(cegui_absdim(/*PixelAligned(*/posChange/*)*/),
			cegui_absdim(/*PixelAligned(*/offset.d_y * 0/*)*/));

		splitter->setPosition(splitter->getPosition() + uOffset);



// 		std::cout << "win pos, ";
// 
// 		if (prev)
// 			std::cout << "prev: " << prev->getXPosition().d_offset;
// 		if (next)
// 			std::cout << "next: " << next->getXPosition().d_offset;
// 		std::cout << "\n";


		//bNoRelayout = false;
	}
}