/***********************************************************************
created:	08 June 2016
author:		Sviatoslav Kokurin

purpose:	Widget for resizable Horizontal Layout Container
*************************************************************************/
/***************************************************************************
*	Code is somewhat based on code of basic CEGUI widgets like Titlebar
*	and FrameWindow, and existing Layout Containers. So I guess it uses
*	the same license as they do.
***************************************************************************/

#pragma once


//CEGUI headers
#include <CEGUI/CEGUI.h>


// Start of CEGUI namespace section
namespace CEGUI
{
	class HSplitter;

	class HLayoutContainer : public SequentialLayoutContainer
	{
	public:
		//! Namespace for global events
		static const String EventNamespace;
		static const String WidgetTypeName;             //!< Window factory name


		/*************************************************************************
		Construction / Destruction
		*************************************************************************/
		/*!
		\brief
		Constructor for Titlebar base class.
		*/
		HLayoutContainer(const String& type, const String& name);

		void offsetSplitter(HSplitter* splitter, const Vector2f& offset);

		/*!
		\brief
		Destructor for Titlebar base class.
		*/
		virtual ~HLayoutContainer(void);


		/// @copydoc LayoutContainer::layout
		virtual void layout();


		/*!
		\brief
		Swaps windows at given positions
		*/
		virtual void swapChildPositions(size_t wnd1, size_t wnd2);



		/*!
		\brief
		Moves a window that is already child of this layout container
		to given position (if the window is currently in a position
		that is smaller than given position, given position is
		automatically decremented
		*/
		virtual void moveChildToPosition(Window* wnd, size_t position);


		Window* getLastWindow();
		HSplitter* getLastSplitter();

		float getChildrenMinWidth();

	protected:

		/*!
		\brief
		Handler called when children of this window gets rearranged in any way

		\param e
		WindowEventArgs object whose 'window' field is set this layout
		container.
		*/
		//virtual void onChildOrderChanged(WindowEventArgs& e);


		virtual void handleSized(const EventArgs& e);

		//our implementation of addChild to add extra splitters when
		//user adds new window
		virtual void addChild_impl(Element* element);

		//our implementation of removeChild so we can remove unneeded splitters
		//when user removes a window
		virtual void removeChild_impl(Element* element);

	private:

		float splitterWidth;

		bool bAddingSplitter = false;

		bool bNoRelayout = false;

		float windowMinSize = 50.0f;
	};

} // End of  CEGUI namespace section