

#include <iostream>

#include "HSplitter.h"
#include "HLayoutContainer.h"



// Start of CEGUI namespace section
namespace CEGUI
{
	const String HSplitter::EventNamespace("HSplitter");
	const String HSplitter::WidgetTypeName("CEGUI/HSplitter");

	HSplitter::HSplitter(const String& type, const String& name) :
		Window(type, name)
	{
		//addTitlebarProperties();
		setAlwaysOnTop(true);

		// basic initialisation
		d_dragging = false;
		d_dragEnabled = true;

		prevSplitter=0;
		nextSplitter=0;

		prevWindow=0;
		nextWindow=0;

		 d_ewSizingCursor = 0;
	}

	/*************************************************************************
	Destructor
	*************************************************************************/
	HSplitter::~HSplitter(void)
	{
	}



	/*HSplitter* HSplitter::getPrevSplitter()
	{
		std::cout << "get prev splitter\n";
		return prevSplitter;
	}

	HSplitter* HSplitter::getNextSplitter()
	{
		std::cout << "get next splitter\n";
		return nextSplitter;
	}*/

	Window* HSplitter::getPrevWindow()
	{
		return prevWindow;
	}

	Window* HSplitter::getNextWindow()
	{
		return nextWindow;
	}

	/*void HSplitter::setPrevSplitter(HSplitter* prev)
	{
		prevSplitter = prev;
	}

	void HSplitter::setNextSplitter(HSplitter* next)
	{
		prevSplitter = next;
	}*/

	void HSplitter::setPrevWindow(Window* prev)
	{
		prevWindow = prev;
	}

	void HSplitter::setNextWindow(Window* next)
	{
		nextWindow = next;
	}

	void HSplitter::setEWSizingCursorImage(const String& name)
	{
		d_ewSizingCursor = &ImageManager::getSingleton().get(name);
	}

	const Image* HSplitter::getEWSizingCursorImage() const
	{
		return d_ewSizingCursor;
	}

	/*************************************************************************
	Handler for mouse movement events
	*************************************************************************/
	void HSplitter::onMouseMove(MouseEventArgs& e)
	{
		// Base class processing.
		Window::onMouseMove(e);

		if (d_dragging && (d_parent != 0))
		{
			Vector2f delta(CoordConverter::screenToWindow(*this, e.position));

			// calculate amount that window has been moved
			delta -= d_dragPoint;

			// move the splitter.  *** Again: splitter objects should only be attached to HLayoutContainer. ***
			((HLayoutContainer*)d_parent)->offsetSplitter(this,delta);

// 			UVector2 uOffset(cegui_absdim(/*PixelAligned(*/delta.d_x/*)*/),
// 				cegui_absdim(/*PixelAligned(*/delta.d_y*0/*)*/));
// 
// 			setPosition(d_area.getPosition() + uOffset);

			++e.handled;
		}

		if (d_ewSizingCursor)
			getGUIContext().getMouseCursor().setImage(d_ewSizingCursor);
	}


	/*************************************************************************
	Handler for mouse button press events
	*************************************************************************/
	void HSplitter::onMouseButtonDown(MouseEventArgs& e)
	{
		// Base class processing
		Window::onMouseButtonDown(e);

		if (e.button == LeftButton)
		{
			if ((d_parent != 0) && d_dragEnabled)
			{
				// we want all mouse inputs from now on
				if (captureInput())
				{
					// initialise the dragging state
					d_dragging = true;
					d_dragPoint = CoordConverter::screenToWindow(*this, e.position);

					// store old constraint area
					d_oldCursorArea = getGUIContext().
						getMouseCursor().getConstraintArea();

					// setup new constraint area to be the intersection of the old area and our grand-parent's clipped inner-area
					Rectf constrainArea;

					if ((d_parent == 0) || (getParent()->getParent() == 0))
					{
						Rectf screen(Vector2f(0, 0), getRootContainerSize());
						constrainArea = screen.getIntersection(d_oldCursorArea);
					}
					else
					{
						constrainArea = getParent()->getParent()->getInnerRectClipper().getIntersection(d_oldCursorArea);
					}

					getGUIContext().getMouseCursor().
						setConstraintArea(&constrainArea);
				}
			}

			++e.handled;
		}
	}


	/*************************************************************************
	Handler for mouse button release events
	*************************************************************************/
	void HSplitter::onMouseButtonUp(MouseEventArgs& e)
	{
		// Base class processing
		Window::onMouseButtonUp(e);

		if (e.button == LeftButton)
		{
			releaseInput();
			++e.handled;
		}

	}

	/*************************************************************************
	Handler for if the window loses capture of the mouse.
	*************************************************************************/
	void HSplitter::onCaptureLost(WindowEventArgs& e)
	{
		// Base class processing
		Window::onCaptureLost(e);

		// when we lose out hold on the mouse inputs, we are no longer dragging.
		d_dragging = false;

		// restore old constraint area
		getGUIContext().
			getMouseCursor().setConstraintArea(&d_oldCursorArea);
	}


	/*************************************************************************
	Handler for when the font for this Window is changed
	*************************************************************************/
	void HSplitter::onFontChanged(WindowEventArgs& e)
	{
		Window::onFontChanged(e);

		if (d_parent)
		{
			getParent()->performChildWindowLayout();
		}
	}


	/*************************************************************************
	Add title bar specific properties
	*************************************************************************/
	void HSplitter::addSplitterProperties(void)
	{
		const String& propertyOrigin = WidgetTypeName;

		//this->addpr

// 		CEGUI_DEFINE_PROPERTY(Window, Image*,
// 			"EWSizingCursorImage", "Property to get/set the E-W (left-right) sizing cursor image for the Splitter. Value should be \"set:[imageset name] image:[image name]\".",
// 			&HSplitter::setEWSizingCursorImage, &HSplitter::getEWSizingCursorImage, 0
// 			);


	//	CEGUI_DEFINE_PROPERTY(HSplitter, bool,
	//		"DraggingEnabled", "Property to get/set the state of the dragging enabled setting for the Titlebar.  Value is either \"true\" or \"false\".",
	//		&HSplitter::setDraggingEnabled, &HSplitter::isDraggingEnabled, true
	//		);
	}



}