/***********************************************************************
created:	08 June 2016
author:		Sviatoslav Kokurin

purpose:	Widget for resizing Horizontal Layout
*************************************************************************/
/***************************************************************************
*	Code is somewhat based on code of basic CEGUI widgets like Titlebar
*	and FrameWindow. So I guess it uses the same license as they do.
***************************************************************************/

#pragma once


//CEGUI headers
#include <CEGUI/CEGUI.h>


// Start of CEGUI namespace section
namespace CEGUI
{
	class HSplitter : public Window
	{
	public:
		static const String EventNamespace;				//!< Namespace for global events
		static const String WidgetTypeName;             //!< Window factory name


		/*************************************************************************
		Construction / Destruction
		*************************************************************************/
		/*!
		\brief
		Constructor for Titlebar base class.
		*/
		HSplitter(const String& type, const String& name);


		/*!
		\brief
		Destructor for Titlebar base class.
		*/
		virtual ~HSplitter(void);

		//we don't really need these functions for now
		/*HSplitter* getPrevSplitter();
		HSplitter* getNextSplitter();
		void setPrevSplitter(HSplitter* prev);
		void setNextSplitter(HSplitter* next);*/

		Window* getPrevWindow();
		Window* getNextWindow();


		void setPrevWindow(Window* prev);
		void setNextWindow(Window* next);




		/*!
		\brief
		Set the image to be used for the east-west sizing mouse cursor.

		\param name
		String holding the name of the Image to be used.

		\return
		Nothing.

		\exception UnknownObjectException thrown if either \a imageset or \a image refer to non-existant entities.
		*/
		void setEWSizingCursorImage(const String& name);

		/*!
		\brief
		Return a pointer to the currently set Image to be used for the east-west
		sizing mouse cursor.

		\return
		Pointer to an Image object, or 0 for none.
		*/
		const Image* getEWSizingCursorImage() const;


	protected:
		/*************************************************************************
		Overridden event handler functions
		*************************************************************************/
		virtual void	onMouseMove(MouseEventArgs& e);
		virtual void	onMouseButtonDown(MouseEventArgs& e);
		virtual void	onMouseButtonUp(MouseEventArgs& e);
		//virtual void	onMouseDoubleClicked(MouseEventArgs& e);
		virtual void	onCaptureLost(WindowEventArgs& e);
		virtual void	onFontChanged(WindowEventArgs &e);


		/*************************************************************************
		Implementation Data
		*************************************************************************/
		bool d_dragging;			//!< set to true when the window is being dragged.
		Vector2f d_dragPoint;		//!< Point at which we are being dragged.
		bool d_dragEnabled;		//!< true when dragging for the widget is enabled.

		Rectf d_oldCursorArea;	//!< Used to backup cursor restraint area.

	private:
		/*************************************************************************
		Private methods
		*************************************************************************/
		void	addSplitterProperties(void);

		//pointer to adjacent splitters
		HSplitter* prevSplitter;
		HSplitter* nextSplitter;

		//pointer to adjacent windows
		Window* prevWindow;
		Window* nextWindow;

		const Image*	d_ewSizingCursor;		//!< East/West sizing cursor image.
	};

} // End of  CEGUI namespace section